###################### R function: covParmsCIs1TermLogis #######################

# For obtaining natural 1-term confidence intervals for the standard deviation 
# and correlation parameters of the random effects for binary-logistic 
# generalised linear mixed models with bivariate random effects.

# Last changed: 26 JUN 2023

covParmsCIs1TermLogis <- function(SigmaHat,m,n,alpha=0.05)
{
  # Obtain the required percentile value:
  
  qVal <- qnorm(1 - 0.5*alpha)
  
  # Obtain the standard deviation and correlation estimtes:
  
  sigma1Hat <- sqrt(SigmaHat[1,1])
  sigma2Hat <- sqrt(SigmaHat[2,2])
  rhoHat <- SigmaHat[1,2]/(sigma1Hat*sigma2Hat)
  
  # Obtain the one-term confidence intervals
  
  facSig1and2 <- exp(qVal/sqrt(2*m))
  CIlowSig1 <- sigma1Hat/facSig1and2
  CIuppSig1 <- sigma1Hat*facSig1and2
  CIlowSig2 <- sigma2Hat/facSig1and2
  CIuppSig2 <- sigma2Hat*facSig1and2
  
  seRhoHat <- qVal/sqrt(m)
  
  CIlowRho <- tanh(atanh(rhoHat) - seRhoHat)
  CIuppRho <- tanh(atanh(rhoHat) + seRhoHat)
  
  # Return the confidence intervals:
  
  return(list(sigma1=c(CIlowSig1,CIuppSig1),sigma2=c(CIlowSig2,CIuppSig2),
              rho=c(CIlowRho,CIuppRho)))
}

######################### End of covParmsCIs1TermLogis #########################
