###################### R function: covParmsCIs2TermLogis #######################

# For obtaining natural 2-term confidence intervals for the standard deviation 
# and correlation parameters of the random effects for binary-logistic 
# generalised linear mixed models with bivariate random effects.

# Last changed: 26 JUN 2023

covParmsCIs2TermLogis <- function(SigmaHat,AsyCov,m,n,alpha=0.05)
{
  # Obtain the required percentile value:
  
  qVal <- qnorm(1 - 0.5*alpha)
  
  # Obtain the standard deviation and correlation estimtes:
  
  sigma1Hat <- sqrt(SigmaHat[1,1])
  sigma2Hat <- sqrt(SigmaHat[2,2])
  rhoHat <- SigmaHat[1,2]/(sigma1Hat*sigma2Hat)
  
  # Obtain the required derivative matrix:
  
  rhoFac <- rhoHat*rhoHat - 1
  K1 <- cbind(c(0.5*sigma2Hat*sigma2Hat*rhoFac,0,
                0.5*rhoHat*sigma2Hat*sigma2Hat),
              c(0,0,(-sigma1Hat*sigma2Hat)),
              c(0,0.5*sigma1Hat*sigma1Hat*rhoFac,
                0.5*rhoHat*sigma1Hat*sigma1Hat))
  K1 <- K1/(sigma1Hat*sigma1Hat*sigma2Hat*sigma2Hat*rhoFac)
  
  # Obtain the two-term estimated asymptotic covariance matrix:
  
  K2 <- AsyCov 
  
  # Obtain the estimated asymptotic covariance matrix of
  # the "naturally" transformed (sigma1,sigma2,rho) vector:
  
  K3 <- tcrossprod(crossprod(t(K1),K2),K1)
  
  # Obtain the required asymptotic standard deviation quantities:
  
  kappa4 <- sqrt(K3[1,1]) ; kappa5 <- sqrt(K3[2,2]) 
  kappa6 <- sqrt(K3[3,3]) 
  
  # Obtain the required confidence interval limits:
  
  kappa7 <- sigma1Hat/exp(qVal*kappa4)
  kappa8 <- sigma1Hat*exp(qVal*kappa4)
  
  kappa9  <- sigma2Hat/exp(qVal*kappa5)
  kappa10 <- sigma2Hat*exp(qVal*kappa5)
  
  kappa11 <- tanh(atanh(rhoHat) - qVal*kappa6)
  kappa12 <- tanh(atanh(rhoHat) + qVal*kappa6)
  
  # Return the confidence intervals:
  
  return(list(sigma1=c(kappa7,kappa8),sigma2=c(kappa9,kappa10),
              rho=c(kappa11,kappa12)))
}

######################### End of covParmsCIs2TermLogis #########################
