########################## R function: generExampData ##########################

# For generation of example generalized linear mixed model data.

# Last changed: 26 JUN 2023

generExampData <- function(m,nVec,XAdistn,XBdistn,betaATrue,betaBTrue,SigmaTrue,family)
{
   # Generate data:

   dA <- length(betaATrue)
   dB <- length(betaBTrue)
   Uvec <- mvrnorm(m,rep(0,dA),SigmaTrue)
   y <- NULL ; XA <- NULL ; XB <- NULL ; idNum <- NULL
   
   for (i in 1:m)
   {
      XAcurr <- matrix(1,nVec[i],dA) 
      if (XAdistn=="norm") XAcurr[,-1] <- matrix(rnorm((dA-1)*nVec[i]),nVec[i],(dA-1))   
      if (XAdistn=="unif") XAcurr[,-1] <- matrix(runif((dA-1)*nVec[i],-1,1),nVec[i],(dA-1))
      if (XBdistn=="norm") XBcurr <- matrix(rnorm(dB*nVec[i]),nVec[i],dB)   
      if (XBdistn=="unif") XBcurr <- matrix(runif(dB*nVec[i],-1,1),nVec[i],dB)
      if (family=="binomial")
      {
         muCurr <- plogis(as.vector(XAcurr%*%(betaATrue + Uvec[i,])) 
                          + as.vector(XBcurr%*%betaBTrue))
         yCurr <- rbinom(nVec[i],1,muCurr)
      }
      if (family=="poisson")
      {
         muCurr <- exp(as.vector(XAcurr%*%(betaATrue + Uvec[i,])) 
                       + as.vector(XBcurr%*%betaBTrue))
         yCurr <- rpois(nVec[i],muCurr)
      }
      XA <- rbind(XA,XAcurr)
      XB <- rbind(XB,XBcurr) 
      y <- c(y,yCurr)
      idNum <- c(idNum,rep(i,nVec[i]))
   }

   # Return the generated data as a list:

   return(list(y=y,idNum=idNum,XA=XA,XB=XB))
}

############################# End of generExampData ############################
